package gs.datagen.v0

/** Type class for data which can be generated.
  */
trait Generated[A]:
  def generate(): A

object Generated:

  def apply[A](
    using
    G: Generated[A]
  ): Generated[A] = G

  final class FromGenerator[A](gen: Gen[A]) extends Generated[A]:
    override def generate(): A = gen.gen()

  /** Used to produce instances of the `Generated` type class.
    *
    * {{{
    * case class MyThing(x: String, y: Int, z: UUID)
    *
    * given genMyThing: Generated[MyThing] =
    *   Gen.of {
    *     for
    *       x <- Gen.string.alpha(Size.fixed(5))
    *       y <- Gen.integer.positive()
    *       z <- Gen.uuid.random()
    *     yield
    *       MyThing(x, y, z)
    *   }
    * }}}
    */
  def of[A](g: Gen[A]): Generated[A] = new FromGenerator[A](g)

end Generated
