package gs.datagen.v0.generators

import gs.datagen.v0.Gen
import scala.concurrent.duration.FiniteDuration
import scala.concurrent.duration.TimeUnit

final class GenFiniteDuration private (
  val lowerBound: Long,
  val upperBound: Long,
  val timeUnit: TimeUnit
) extends Gen[FiniteDuration]:

  override def generate(input: Any): FiniteDuration =
    FiniteDuration(Gen.rng().nextLong(lowerBound, upperBound + 1), timeUnit)

object GenFiniteDuration:

  def apply(
    lowerBound: Long,
    upperBound: Long,
    timeUnit: TimeUnit
  ): GenFiniteDuration =
    if lowerBound < 0 then
      throw new IllegalArgumentException(
        s"The lower bound of a finite duration must be greater than or equal to 0. Received: '$lowerBound'"
      )
    else if lowerBound > upperBound then
      throw new IllegalArgumentException(
        s"The lower bound of a finite duration must less than the upper bound. Received lowerBound='$lowerBound' and upperBound='$upperBound'"
      )
    else new GenFiniteDuration(lowerBound, upperBound, timeUnit)

end GenFiniteDuration
