package gs.datagen.v0.generators

import gs.datagen.v0.Gen

final class GenInteger private (
  val lowerBound: Int,
  val upperBound: Int
) extends Gen[Int]:

  override def generate(input: Any): Int =
    Gen.rng().nextInt(lowerBound, upperBound + 1)

object GenInteger:

  def apply(
    lowerBound: Int,
    upperBound: Int
  ): GenInteger =
    if upperBound < lowerBound then new GenInteger(upperBound, lowerBound)
    else new GenInteger(lowerBound, upperBound)

  def positive(): GenInteger =
    GenInteger(1, Int.MaxValue)

  def negative(): GenInteger =
    GenInteger(Int.MinValue, 0)

  def nonNegative(): GenInteger =
    GenInteger(0, Int.MaxValue)

  def inRange(
    lower: Int,
    upper: Int
  ): GenInteger =
    GenInteger(lower, upper)

end GenInteger
