package gs.datagen.v0.generators

import gs.datagen.v0.Gen

final class GenLong private (
  val lowerBound: Long,
  val upperBound: Long
) extends Gen[Long]:

  override def generate(input: Any): Long =
    Gen.rng().nextLong(lowerBound, upperBound + 1)

object GenLong:

  def apply(
    lowerBound: Long,
    upperBound: Long
  ): GenLong =
    if upperBound < lowerBound then new GenLong(upperBound, lowerBound)
    else new GenLong(lowerBound, upperBound)

  def positive(): GenLong =
    GenLong(1, Long.MaxValue)

  def negative(): GenLong =
    GenLong(Long.MinValue, 0)

  def nonNegative(): GenLong =
    GenLong(0, Long.MaxValue)

  def inRange(
    lower: Long,
    upper: Long
  ): GenLong =
    GenLong(lower, upper)

end GenLong
