package gs.datagen.v0.generators

import gs.datagen.v0.Gen
import scala.reflect.ClassTag

final class GenOneOf[A: ClassTag] private (
  val choices: Array[A]
) extends Gen[A]:

  override def generate(input: Any): A =
    choices(Gen.rng().nextInt(0, choices.length))

object GenOneOf:

  def choices[A: ClassTag](
    cs: A*
  ): GenOneOf[A] =
    if cs.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOf[A](cs.toArray)

  def list[A: ClassTag](
    choices: List[A]
  ): GenOneOf[A] =
    if choices.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOf[A](choices.toArray)

  def set[A: ClassTag](
    choices: Set[A]
  ): GenOneOf[A] =
    if choices.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOf[A](choices.toArray)

end GenOneOf
