package gs.datagen.v0.generators

import gs.datagen.v0.Gen
import scala.reflect.ClassTag

final class GenOneOfGen[A: ClassTag] private (
  val choices: Array[Gen[A]]
) extends Gen[A]:

  override def generate(input: Any): A =
    choices(Gen.rng().nextInt(0, choices.length)).gen()

object GenOneOfGen:

  def choices[A: ClassTag](
    cs: Gen[A]*
  ): GenOneOfGen[A] =
    if cs.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOfGen[A](cs.toArray)

  def list[A: ClassTag](
    choices: List[Gen[A]]
  ): GenOneOfGen[A] =
    if choices.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOfGen[A](choices.toArray)

  def set[A: ClassTag](
    choices: Set[Gen[A]]
  ): GenOneOfGen[A] =
    if choices.isEmpty then
      throw new IllegalArgumentException(
        "At least one choice must be provided."
      )
    else new GenOneOfGen[A](choices.toArray)

end GenOneOfGen
