package gs.datagen.v0.generators

import GenOption.DeterminationType
import gs.datagen.v0.Gen

final class GenOption[A](
  val determinationType: DeterminationType,
  val generator: Gen[A]
) extends Gen[Option[A]]:

  override def generate(input: Any): Option[A] =
    determinationType match
      case DeterminationType.AlwaysSome =>
        Some(generator.gen())
      case DeterminationType.AlwaysNone =>
        None
      case DeterminationType.Random =>
        if Gen.rng().nextBoolean() then Some(generator.gen()) else None

object GenOption:

  def apply[A](generator: Gen[A]): GenOption[A] =
    new GenOption[A](DeterminationType.Random, generator)

  def random[A](generator: Gen[A]): GenOption[A] = apply[A](generator)

  def alwaysSome[A](generator: Gen[A]): GenOption[A] =
    new GenOption[A](DeterminationType.AlwaysSome, generator)

  def alwaysNone[A](generator: Gen[A]): GenOption[A] =
    new GenOption[A](DeterminationType.AlwaysNone, generator)

  enum DeterminationType:
    case AlwaysSome, AlwaysNone, Random

  object DeterminationType:

    given CanEqual[DeterminationType, DeterminationType] = CanEqual.derived

  end DeterminationType

end GenOption
