package gs.datagen.v0.generators

import java.util.Random

sealed trait MinMax:
  def min: Int
  def max: Int

  def select(random: Random): Int =
    if min == 0 && max == 0 then 0
    else random.nextInt(min, max)

object MinMax:

  val Zero: MinMax.NonNegative = NonNegative(0, 0)

  def apply(
    min: Int,
    max: Int
  ): MinMax = Generic(min, max)

  def nonNegative(
    min: Int,
    max: Int
  ): MinMax.NonNegative =
    NonNegative(min, max)

  final class Generic private (
    val min: Int,
    val max: Int
  ) extends MinMax

  object Generic:

    def apply(
      min: Int,
      max: Int
    ): Generic =
      if min <= max then new Generic(min, max)
      else new Generic(max, min)

  end Generic

  final class NonNegative private (
    val min: Int,
    val max: Int
  ) extends MinMax

  object NonNegative:

    def apply(
      min: Int,
      max: Int
    ): NonNegative =
      if min < 0 || max < 0 then
        throw new IllegalArgumentException(
          "Only values >= 0 are allowed in this min/max pair."
        )
      else if min <= max then new NonNegative(min, max)
      else new NonNegative(max, min)

  end NonNegative

end MinMax
