package gs.datagen.v0

import Datagen.NoInput

class DatagenTests extends munit.FunSuite:

  test("Datagen.any should lift a pure value") {
    val data = "foo"
    val dg   = Datagen.pure(data)
    assert(dg.generate(()) == data)
  }

  test("Datagen.NoInput should support flatMap and map") {
    val data = "foo"

    val p1: NoInput[String] =
      Datagen.pure(data).flatMap(x => Datagen.pure(data))

    val p2: Datagen[String, Any] =
      Datagen.pure(data).map(x => data)

    assert(p1.generate(()) == data)
    assert(p2.generate(()) == data)
  }

  test(
    "Datagen.NoInput should support for-comprehensions with no input in the yield"
  ) {
    val data = "foo"

    val program =
      for
        x <- Datagen.pure(data)
        y <- Datagen.pure(data)
        z <- Datagen.pure(data)
      yield (x, y, z)

    assert(program.generate(()) == (data, data, data))
  }

  test(
    "Datagen.NoInput should support for-comprehensions with input in the yield"
  ) {
    val data = "foo"

    val program =
      for
        x <- Datagen.pure(data)
        y <- Datagen.pure(data)
        z <- Datagen.pure(data)
      yield (w: String) => (x, y, z, w)

    assert(program.generate(data) == (data, data, data, data))
  }

end DatagenTests
